﻿using iFileProxy.Config;
using Serilog;
using System.Data;
using System.Text.Json;
using MySql.Data.MySqlClient;
using iFileProxy.Models;

namespace iFileProxy.Helpers
{
    public class DatabaseHelper
    {
        Database _db;
        AppConfig _appConfig;
        private readonly static Serilog.ILogger _logger = Log.Logger.ForContext<DatabaseHelper>();

        Dictionary<string, DB> _dbDictionary = new Dictionary<string, DB>();

        public DatabaseHelper(AppConfig appConfig)
        {
            _logger.Information("Initializing DatabaseHelper...");
            _db = appConfig.Database;
            _appConfig = appConfig;
            try
            {
                _logger.Information("Done.");
            }
            catch (Exception e)
            {
                _logger.Fatal($"程序异常: {e.Message}");
            }
            LoadDbDict();
        }

        /// <summary>
        /// 加载数据库描述字典
        /// </summary>
        public void LoadDbDict()
        {
            foreach (DB item in _db.Databases)
            {
                _dbDictionary.Add(item.Description, item);
                _logger.Debug($"Db Config: {item.Description} <=> {item.DatabaseName} Loaded.");
            }
        }
        /// <summary>
        /// 获取一个指定数据库的连接
        /// </summary>
        /// <param name="db_desc">数据库描述字段 对应AppConfig的description字段</param>
        /// <returns></returns>
        /// <exception cref="Exception">若某些不允许为空的字段出现空值 则抛出此异常</exception>
        /// 
        public MySqlConnection GetDBConn(string db_desc)
        {
            if (!_dbDictionary.TryGetValue(db_desc, out DB Db))
            {
                throw new Exception($"未找到与 {db_desc} 相匹配的数据库配置");
            }

            var db_user = Db.User ?? _db.Common.User;
            var db_password = Db.Password ?? _db.Common.Password;
            var db_host = Db.Host ?? _db.Common.Host;
            var db_port = Db.Port ?? _db.Common.Port;

            if (db_user == null || db_password == null || db_host == null || db_port == null)
                throw new NoNullAllowedException("数据库配置获取失败,不允许为空的字段出现空值");

            string db_connstr = $"server={db_host};user={db_user};database={Db.DatabaseName};port={db_port};password={db_password};Pooling=true;MaximumPoolSize=500;";
            MySqlConnection conn;
            try
            {
                conn = new MySqlConnection(db_connstr);
                conn.Open();
            }
            catch (Exception ex)
            {
                _logger.Fatal($"获取Mysql连接时出现异常:{ex.Message}");
                throw;
            }
            return conn;
        }
        /// <summary>
        /// 获取一个json格式的数据表
        /// </summary>
        /// <param name="sql"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static string GetTableData(string sql, MySqlConnection conn)
        {
            DataTable dataTable = new DataTable();

            using (MySqlCommand queryAllUser = new MySqlCommand(sql, conn))
            {
                using (MySqlDataAdapter adapter = new MySqlDataAdapter(queryAllUser))
                    adapter.Fill(dataTable);
            }
            return JsonSerializer.Serialize(dataTable);

        }

        public bool InsertTaskData(TaskInfo taskInfo)
        {
            _logger.Debug(JsonSerializer.Serialize(taskInfo));
            string sql = "INSERT INTO `t_tasks_info` (`tid`, `file_name`, `client_ip`, `add_time`, `update_time`, `status`, `url`, `size`, `hash`) " +
                "VALUES (@tid, @file_name, @client_ip, @add_time, @update_time, @status, @url, @size, @hash)";
            MySqlConnection conn = GetDBConn("iFileProxy_Db");

            try
            {
                using MySqlCommand sqlCmd = new MySqlCommand(sql, conn);
                sqlCmd.Parameters.AddWithValue("@tid", taskInfo.TaskId);
                sqlCmd.Parameters.AddWithValue("@file_name", taskInfo.FileName);
                sqlCmd.Parameters.AddWithValue("@client_ip", taskInfo.ClientIp);
                sqlCmd.Parameters.AddWithValue("@add_time", taskInfo.AddTime.ToString("yyyy-MM-dd HH:mm:ss"));
                sqlCmd.Parameters.AddWithValue("@update_time", taskInfo.UpdateTime.ToString("yyyy-MM-dd HH:mm:ss"));
                sqlCmd.Parameters.AddWithValue("@status", taskInfo.Status);
                sqlCmd.Parameters.AddWithValue("@url", taskInfo.Url);
                sqlCmd.Parameters.AddWithValue("@size", taskInfo.Size);
                sqlCmd.Parameters.AddWithValue("@hash", taskInfo.Hash);
                
                sqlCmd.ExecuteNonQuery();
            }
            catch (Exception)
            {
                _logger.Fatal($"插入数据时出现问题");
                throw;
            }
            finally
            { 
                conn.Close();
            }
            return true;


        }

        public void TryInitialDB()
        {
            string sql = "ALTER TABLE `t_region_config`  ADD COLUMN `stop_server_info_str` varchar(255) CHARACTER SET utf8mb4 NOT NULL AFTER `stop_server_config_str`";
            MySqlConnection conn = GetDBConn("deploy_config");

            try
            {
                using MySqlCommand cmd = new(sql, conn);
                cmd.ExecuteNonQuery();
            }
            catch { }
            finally
            {
                conn.Close();
            }
        }
    }
}
