﻿using iFileProxy.Config;
using iFileProxy.Helpers;
using iFileProxy.Models;
using Serilog;
using System.Diagnostics;
using System.Security.Policy;

namespace iFileProxy.Services
{
    /// <summary>
    /// 下载任务管理器
    /// </summary>
    public class TaskManager
    {
        private readonly static Serilog.ILogger _logger = Log.Logger.ForContext<TaskManager>();
        private readonly AppConfig? _appConfig = AppConfig.GetCurrConfig("iFileProxy.json");
        private readonly DatabaseHelper _dbHelper;
        private Dictionary<string, TaskInfo> runningTasks = [];
        public TaskManager() 
        {
            _logger.Information("Initializing TaskManager...");
            if (_appConfig != null)
                _dbHelper = new DatabaseHelper(_appConfig);
            else
            {
                _logger.Fatal($"Failed to load application configuration");
                Environment.Exit(1);
            }
            _logger.Information("TaskManager init succ.");
        }
        /// <summary>
        /// 添加一个新的下载任务
        /// </summary>
        /// <param name="c">HttpContext</param>
        /// <returns></returns>
        public TaskAddState AddTask(HttpContext c) 
        {
            // 尝试从 X-Forwarded-For 请求头获取客户端IP地址
            string? clientIp = c.Request.Headers["X-Forwarded-For"].FirstOrDefault();
            // 如果 X-Forwarded-For 头不存在，回退到 RemoteIpAddress
            if (string.IsNullOrEmpty(clientIp))
            {
                clientIp = c.Connection.RemoteIpAddress?.ToString();
            }
            string? t_url = c.Request.Query["url"].FirstOrDefault();

            foreach (var t in runningTasks)
            {
                if (t.Value.Url == t_url)
                { 
                    return TaskAddState.ErrUrlRepeat;
                }
            }

            if (!MasterHelper.CheckUrlIsValid(t_url))
                return TaskAddState.ErrUrlInvalid;

            DownloadFileInfo fileInfo = FileDownloadHelper.GetDownloadFileInfo(t_url);
            TaskInfo taskInfo = new()
            {
                Url = t_url,
                TaskId = Guid.NewGuid().ToString(),
                AddTime = DateTime.Now,
                FileName = fileInfo.FileName,
                ClientIp = clientIp,
                Size = fileInfo.Size,
                Status = TaskState.Running,
                UpdateTime = DateTime.Now
            } ;
            if (_dbHelper.InsertTaskData(taskInfo))
            {
                StartTask(taskInfo);
                _logger.Debug("数据插入成功");
                return TaskAddState.Success;
            }
            else
                return TaskAddState.ErrDbFail;
        }

        public async void StartTask(TaskInfo task_info)
        {
            if (runningTasks.ContainsKey(task_info.TaskId))
            {
                _logger.Error($"指定的task已经存在!!!");
                return;
            }    
            Process aria2c = new();
            aria2c.StartInfo = new ProcessStartInfo
            {
                FileName = _appConfig.DownloadOptions.Aria2cPath,
                WorkingDirectory = _appConfig.DownloadOptions.SavePath,
                Arguments = $"-x {_appConfig.DownloadOptions.ThreadNum} -s {_appConfig.DownloadOptions.ThreadNum} {task_info.Url}",
                RedirectStandardOutput = true ,
                RedirectStandardError = true ,
                RedirectStandardInput = true ,
                UseShellExecute = false,
                Environment = { { "TaskId", task_info.TaskId } } 
            };
            try
            {
                aria2c.Start();
                aria2c.BeginOutputReadLine();
                aria2c.BeginErrorReadLine();
                aria2c.OutputDataReceived += Aria2c_OutputDataReceived;
                aria2c.ErrorDataReceived += Aria2c_ErrorDataReceived;
                runningTasks.Add(task_info.TaskId, task_info);
                await aria2c.WaitForExitAsync();
                if (aria2c.ExitCode != 0)
                    _logger.Error($"task: {task_info.TaskId} 进程退出状态异常 ExitCode: {aria2c.ExitCode}");
                else
                    runningTasks.Remove(task_info.TaskId);
            }
            catch (Exception)
            {
                _logger.Fatal("执行下载任务时候出现致命问题");
                throw;
            }
        }

        private void Aria2c_ErrorDataReceived(object sender, DataReceivedEventArgs e)
        {
            if (e.Data == null || e.Data.Trim() == "")
                return;
            Process c = (Process)sender;
            _logger.Error($"[TaskId: {c.StartInfo.Environment["TaskId"]}] {e.Data}");
        }

        private void Aria2c_OutputDataReceived(object sender, DataReceivedEventArgs e)
        {
            if (e.Data == null || e.Data.Trim() == "")
                return;
            Process c = (Process)sender;
            _logger.Debug($"[TaskId: {c.StartInfo.Environment["TaskId"]}] {e.Data}");
        }

        //public bool DeleteTask(HttpContext c)
        //{

        //}

        //public bool UpdateTaskStatus(HttpContext c)
        //{
        //}
        //public List<TaskInfo> GetAllTaskInfo(HttpContext c)
        //{

        //}

        //public TaskInfo GetTaskInfo(HttpContext c)
        //{

        //}
    }
}
